from django.test import TestCase
from django.core.exceptions import ValidationError
from unittest.mock import patch
from param_field.parser import parse_fields
from param_field.params import ParamDict
from param_field.forms import ParamInputForm
from param_field.params import *

from django.core.files.uploadedfile import SimpleUploadedFile

from decimal import Decimal



class TestParamInputForm(TestCase):

    def test_field_param_validation(self):
        """Test fields call param validation function to validate
        input values"""
        d = ParamDict("""number:Integer->min:33""")

        form_data = {'number': 44 }
        form = ParamInputForm(params=d, data=form_data)
        self.assertTrue(form.is_valid())

        form_data = {'number': 12}
        form = ParamInputForm(params=d, data=form_data)
        self.assertFalse(form.is_valid())
 
        # Test param object validation is called
        with patch.object(IntegerParam, 'validate') as mockval:
            self.assertEqual(mockval.call_count, 0)

            form_data = {'number': 44}
            form = ParamInputForm(params=d, data=form_data)
            self.assertTrue(form.is_valid())

            self.assertEqual(mockval.call_count, 1)

        with patch.object(IntegerParam, 'validate') as mockval:
            mockval.side_effect = ValidationError('')
            self.assertEqual(mockval.call_count, 0)

            form_data = {'number': 44}
            form = ParamInputForm(params=d, data=form_data)
            self.assertFalse(form.is_valid())

            self.assertEqual(mockval.call_count, 1)

    def test_error_message(self):
        """Test ValidationError message is used as the form's
        error message when validation fails"""
        d = ParamDict("""number:Integer->min:10""")
        
        with patch.object(IntegerParam, 'validate') as mockval:
            mockval.side_effect = ValidationError('12345asdf6789')
            self.assertEqual(mockval.call_count, 0)

            form_data = {'number': 44}
            form = ParamInputForm(params=d, data=form_data)
            self.assertFalse(form.is_valid())

            self.assertEqual(mockval.call_count, 1)
            self.assertTrue('12345asdf6789' in form.as_p())

    def test_type_validation(self):
        
        # Integer
        d = ParamDict("number:Integer->min:12 max:32")
        form = ParamInputForm(params=d, data={'number':'20'})
        self.assertTrue(form.is_valid())
        self.assertIsInstance(form.cleaned_data['number'], int)
        self.assertEqual(form.cleaned_data['number'], 20)
 
        # Decimal
        d = ParamDict("number:Decimal->min:12.0 max:32.0")
        form = ParamInputForm(params=d, data={'number':'20'})
        self.assertTrue(form.is_valid())
        self.assertIsInstance(form.cleaned_data['number'], Decimal)
        self.assertEqual(form.cleaned_data['number'], 20)

        # Dimmension
        d = ParamDict("number:Dimmension->min:12.0 max:32.0")
        form = ParamInputForm(params=d, data={'number':'20'})
        self.assertTrue(form.is_valid())
        self.assertIsInstance(form.cleaned_data['number'], Decimal)
        self.assertEqual(form.cleaned_data['number'], 20)

        # Bool
        d = ParamDict("enable:Bool")
        form = ParamInputForm(params=d, data={'enable': 'true'})
        self.assertTrue(form.is_valid())
        self.assertIsInstance(form.cleaned_data['enable'], bool)
        self.assertEqual(form.cleaned_data['enable'], True)

        # Text
        d = ParamDict("name:Text")
        form = ParamInputForm(params=d, data={'name': 'some name'})
        self.assertTrue(form.is_valid())
        self.assertIsInstance(form.cleaned_data['name'], str)
        self.assertEqual(form.cleaned_data['name'], 'some name')

        # TextArea
        d = ParamDict("name:TextArea")
        form = ParamInputForm(params=d, data={'name': 'some name'})
        self.assertTrue(form.is_valid())
        self.assertIsInstance(form.cleaned_data['name'], str)
        self.assertEqual(form.cleaned_data['name'], 'some name')

    def test_required_params(self):
        """Test parameters without a default value are required"""
        # Integer
        d = ParamDict("number:Integer")
        form = ParamInputForm(params=d, data={})
        self.assertFalse(form.is_valid())
        
        d = ParamDict("number:Integer->default:12 required:False")
        form = ParamInputForm(params=d, data={})
        self.assertTrue(form.is_valid())

        d = ParamDict("number:Integer->required:False")
        form = ParamInputForm(params=d, data={})
        self.assertTrue(form.is_valid())
        
        d = ParamDict("number:Integer->required:True")
        form = ParamInputForm(params=d, data={'number': 12})
        self.assertTrue(form.is_valid())
        
        # Decimal
        d = ParamDict("number:Decimal")
        form = ParamInputForm(params=d, data={})
        self.assertFalse(form.is_valid())

        d = ParamDict("number:Decimal->required:False")
        form = ParamInputForm(params=d, data={})
        self.assertTrue(form.is_valid())
 
        d = ParamDict("number:Decimal->required:False")
        form = ParamInputForm(params=d, data={'number': '44'})
        self.assertTrue(form.is_valid())

        # Dimmension
        d = ParamDict("number:Dimmension-> default:23.3")
        form = ParamInputForm(params=d, data={})
        self.assertFalse(form.is_valid())

        d = ParamDict("number:Dimmension->default:13.2 required:False")
        form = ParamInputForm(params=d, data={})
        self.assertTrue(form.is_valid())

        d = ParamDict("number:Dimmension->default:13.2 required:False")
        form = ParamInputForm(params=d, data={'number': '2'})
        self.assertTrue(form.is_valid())
        
        d = ParamDict("number:Dimmension->default:23.3")
        form = ParamInputForm(params=d, data={'number': '2'})
        self.assertTrue(form.is_valid())

        # Text
        d = ParamDict("name: Text")
        form = ParamInputForm(params=d, data={})
        self.assertFalse(form.is_valid())

        d = ParamDict('name: Text->required: False')
        form = ParamInputForm(params=d, data={'text': 'some text'})
        self.assertTrue(form.is_valid())
        
        d = ParamDict('name: Text->required: False default:"2asdf"')
        form = ParamInputForm(params=d, data={'text': 'some text'})
        self.assertTrue(form.is_valid())

        d = ParamDict('name: Text->required: False')
        form = ParamInputForm(params=d, data={'text': 'some text'})
        self.assertTrue(form.is_valid())
        
        # TextArea   
        d = ParamDict("name: TextArea")
        form = ParamInputForm(params=d, data={})
        self.assertFalse(form.is_valid())

        d = ParamDict('name: TextArea->required: False')
        form = ParamInputForm(params=d, data={'text': 'some text'})
        self.assertTrue(form.is_valid())
        
        d = ParamDict('name: TextArea->required: False default:"2asdf"')
        form = ParamInputForm(params=d, data={'text': 'some text'})
        self.assertTrue(form.is_valid())

        d = ParamDict('name: TextArea->required: False')
        form = ParamInputForm(params=d, data={'text': 'some text'})
        self.assertTrue(form.is_valid())

        # Bool
        d = ParamDict("name: Bool")
        form = ParamInputForm(params=d, data={})
        self.assertFalse(form.is_valid())

        d = ParamDict('name: Bool->required: False')
        form = ParamInputForm(params=d, data={})
        self.assertTrue(form.is_valid())
       
        # Files
        d = ParamDict("a_file: File", file_support=True)
        form = ParamInputForm(params= d, data={})
        self.assertFalse(form.is_valid())
        
        d = ParamDict("a_file: File -> required:False", file_support=True)
        form = ParamInputForm(params= d, data={})
        self.assertTrue(form.is_valid())

        d = ParamDict('a_file: File-> required:True', file_support=True)
        form = ParamInputForm(params=d, data ={}, files = {
            'a_file': SimpleUploadedFile('the_file.txt', b"and the content")})
        self.assertTrue(form.is_valid())

        d = ParamDict('a_file: File-> required:False', file_support=True)
        form = ParamInputForm(params=d, data ={}, files = {
            'a_file': SimpleUploadedFile('the_file.txt', b"and the content")})
        self.assertTrue(form.is_valid())



    def test_hidden_params(self):
        """Test invisible params are not present in the form"""
        d = ParamDict("""
                number_holes:Integer->min:33
                enable:Bool->default:False hidden:True""")

        form_data = {'number_holes': 44 }
        form = ParamInputForm(params=d)
        self.assertTrue('number_holes' in form.as_p())
        self.assertTrue('enable' in form.as_p())
    
        form_data = {'number_holes': 44, 'enable':True}
        form = ParamInputForm(params=d, data=form_data)
        self.assertTrue('number_holes' in form.as_p())
        self.assertTrue('enable' in form.as_p())
        self.assertTrue('hidden' in form.as_p())
        self.assertTrue(form.is_valid())
        self.assertEqual(len(form.cleaned_data), 2)
        self.assertEqual(form.cleaned_data['number_holes'], 44)
        self.assertEqual(form.cleaned_data['enable'], True)
        
        form_data = {'number_holes': 44, 'enable':True}
        form = ParamInputForm(params=d, data=form_data)
        self.assertTrue('number_holes' in form.as_p())
        self.assertTrue('enable' in form.as_p())
        self.assertTrue(form.is_valid())
        self.assertEqual(len(form.cleaned_data), 2)
        self.assertEqual(form.cleaned_data['number_holes'], 44)

    def test_labels_from_name(self):
        """Test a label is generated for each field using field name"""
        d = ParamDict("""
                number_of_holes:Integer->min:33
                enable:Bool->default:False hidden:True""")

        form = ParamInputForm(params=d)
        self.assertTrue('Number of holes' in form.as_p())
        self.assertTrue('enable' in form.as_p())
    
    def test_choices_present(self):
        """Test param choices are present in form"""
        d = ParamDict('name:Text->choices:["445566", "yyttuu"]')
        
        form = ParamInputForm(params=d)
        self.assertTrue('445566' in form.as_p())
        self.assertTrue('yyttuu' in form.as_p())

    def test_choices_restriction(self):
        """ """
        # Text
        d = ParamDict('a_name: Text->choices:["aa", "bb", "cc"]')
        
        for s in ('aa', 'bb', 'cc'):
            form = ParamInputForm(params=d, data={'a_name': s})
            self.assertTrue(form.is_valid())

        for s in ('dd', 'ee', 'ff'):
            form = ParamInputForm(params=d, data={'a_name': s})
            self.assertFalse(form.is_valid())
        
        # Integer
        d = ParamDict('holes: Integer->choices: [12, 33, 44]')
       
        for s in (12, 33, 44):
            form = ParamInputForm(params=d, data={'holes': s})
            self.assertTrue(form.is_valid())

        for s in (11, -1, 88):
            form = ParamInputForm(params=d, data={'holes': s})
            self.assertFalse(form.is_valid())
       
        # Decimal
        d = ParamDict('width: Decimal->choices: [23.4, 44.3, 55.3]')

        for s in (Decimal('23.4'), Decimal('44.3'), Decimal('55.3')):
            form = ParamInputForm(params=d, data={'width': s})
            self.assertTrue(form.is_valid())
        
        for s in (Decimal('-23.4'), Decimal('14.3'), Decimal('55.4')):
            form = ParamInputForm(params=d, data={'width': s})
            self.assertFalse(form.is_valid())

        # Dimmension 
        d = ParamDict('width: Dimmension->choices: [23.4, 44.3, 55.3]')

        for s in (Decimal('23.4'), Decimal('44.3'), Decimal('55.3')):
            form = ParamInputForm(params=d, data={'width': s})
            self.assertTrue(form.is_valid())
        
        for s in (Decimal('-23.4'), Decimal('14.3'), Decimal('55.4')):
            form = ParamInputForm(params=d, data={'width': s})
            self.assertFalse(form.is_valid())

    def test_default_present(self):
        """Test default value is present in the rendered form"""
        d = ParamDict('width: Decimal->default: 77.66')
        form = ParamInputForm(params=d)
        self.assertTrue('77.66' in form.as_p())
        
        d = ParamDict('width: Integer->default: 7')
        form = ParamInputForm(params=d)
        self.assertTrue('7' in form.as_p()) 
        
        d = ParamDict('width: Text->default: "just a string"')
        form = ParamInputForm(params=d)
        self.assertTrue('just a string' in form.as_p())

        d = ParamDict('width: Bool->default: True')
        form = ParamInputForm(params=d)
        self.assertTrue('checked' in form.as_p())
        
        d = ParamDict('width: Bool->default: False')
        form = ParamInputForm(params=d)
        self.assertFalse('checked' in form.as_p())



from django.test import TestCase
from django.core.exceptions import ValidationError
from param_field.models import ParamField
from param_field.params import *
from param_field.forms import *



class TestParamField(TestCase):

    def test_parse(self):
        """Test strings are parsed into a ParamDict"""
        pf = ParamField()
        params = "number: Integer->default: 12"
        parsed = pf.clean(params, None)
        self.assertIsInstance(parsed, ParamDict)
        self.assertIsInstance(parsed["number"], IntegerParam)
        self.assertEqual(parsed["number"].default, 12)

    def test_form_construction(self):
        pf = ParamField()
        params = "number: Integer->default: 12"
        parsed = pf.clean(params, None)
        form = parsed.form()
        self.assertIsInstance(form, ParamInputForm)

    def test_file_support(self):
        # TODO: Test against file params when implemented
        pf = ParamField(file_support=False)
        params = "number: Integer->default: 12"
        parsed = pf.clean(params, None)

    # TODO: More test 



from django.test import TestCase
from django.core.exceptions import ValidationError
from param_field.models import ParamField
from param_field.params import *


class TestParamField(TestCase):

    def test_params_are_parsed(self):
        p = ParamField()

        # Empty value
        v = p.clean('', None)
        self.assertEqual(len(v), 0)

        # One liner
        v = p.clean('''lines: Integer-> default:12 max:44 
                label:"number of lines"''', None)
        self.assertIsInstance(v, ParamDict)
        self.assertIsInstance(v['lines'], IntegerParam)
        self.assertEqual(v['lines'].default, 12)
        self.assertEqual(v['lines'].max, 44)
        self.assertEqual(v['lines'].label, 'number of lines')

        with self.assertRaises(ValidationError):
            p.clean('invalid-name:Bool', None)

        with self.assertRaises(ValidationError):
            p.clean('invalid_name:Invalidparam', None)

        with self.assertRaises(ValidationError):
            p.clean('invalid_property:Bool-> color:"red"', None)
        
        with self.assertRaises(ValidationError):
            p.clean('invalid_property_value:Bool-> default:"red"', None)
        
        # Two liner
        v = p.clean("""number_holes:Integer->default:12
                activate: Bool""", None)
        self.assertIsInstance(v, ParamDict)
        self.assertIsInstance(v['number_holes'], IntegerParam)
        self.assertIsInstance(v['activate'], BoolParam)
        self.assertEqual(v['number_holes'].default, 12)


    def test_errors_detected(self):

        p = ParamField()

        with self.assertRaises(ValidationError):
            p.clean('holes: Integer->default:99999999999', None)

        with self.assertRaises(ValidationError):
            p.clean('width: Decimal->default:2', None)

        with self.assertRaises(ValidationError):
            p.clean('widht: Dimmension->default:-3.4', None)



from pyparsing import *
from decimal import Decimal
from collections import OrderedDict
from .params import *

FIELD_TO_PARAM = {
    'Bool': BoolParam,
    'Decimal': DecimalParam,
    'Dimmension': DimmensionParam,
    'Integer': IntegerParam,
    'Text': TextParam,
    'TextArea': TextAreaParam,
    'File': FileParam,
    'Image': ImageParam,
}


reserved_keywords = Keyword("Integer")|Keyword("Bool")|Keyword("Dimmension")\
        |Keyword("Decimal")|Keyword("Text")|Keyword("TextArea")|Keyword("File")\
        |Keyword("Image")|Keyword("default")|Keyword("min_length")|Keyword("max_length")\
        |Keyword("min")|Keyword("max")|Keyword("help_text")|Keyword("label")\
        |Keyword("hidden")|Keyword("odd")|Keyword("even")|Keyword("choices")\
        |Keyword("required")

cvtInt = lambda t: int(t[0])
cvtDec = lambda t: Decimal(t[0])
cvtBool = lambda t: (True if t[0]=="True" else False)

def rangeCheck(minval=None, maxval=None):
    if minval is None and maxval is None:
        # this exception occurs not at parse time, but while defining the grammar
        raise ValueError("minval or maxval must be specified")

    def rangeCheckParseAction(string, loc, tokens):
        parsedval = tokens[0]
        if minval is not None:
            if maxval is not None:
                valid = minval <= parsedval <= maxval
            else:
                valid = minval <= parsedval
        else:
            if maxval is not None:
                valid = parsedval <= maxval

        if not valid:
            err = "value not in range ({},{})".format(minval, maxval)
            raise ParseFatalException(string, loc, err)

    return rangeCheckParseAction    

def lengthCheck(max_length=STRING_MAX_LENGTH):

    def lengthCheckParseAction(string, loc, tokens):
        if len(tokens[0]) > max_length:
            err = "string exceeds maximum length".format(max_length)
            raise ParseFatalException(string, loc, err)

    return lengthCheckParseAction

def propToDict(tokens):
    """Convert field property list to dictionary"""
    prop_dict = OrderedDict()

    for token in tokens:
        prop_dict[token.property_name] = token.property_value

    return prop_dict

def lstToList(tokens):
    l = list()
    
    for token in tokens[0]:
        l.append(token)
    
    return [l]

def fieldToParam(token):
    name, field_type, props = token.field 
    return (name, FIELD_TO_PARAM[field_type](**props))

lowercase = "abcdefghijklmnopqrstuvwxyz"
lowercasenums = "abcdefghijklmnopqrstuvwxyz0123456789"
arithOp = Word("+-*/^", max=1)
lparen,rparen,lbrack,rbrack,lbrace,rbrace = map(Suppress,"()[]{}")
colon = Suppress(":")
comma = Suppress(",")
plusorminus = oneOf("+ -")
arrow = Suppress("->")
number = Word(nums)


# Define data primitives and limits
integer = Combine(Optional(plusorminus)+number)\
    .setName("integer").setParseAction(cvtInt)\
    .addParseAction(rangeCheck(INT_MIN, INT_MAX))
real = Combine(Optional(plusorminus)+number+"."+number)\
    .setName("real").setParseAction(cvtDec)\
    .addParseAction(rangeCheck(REAL_MIN, REAL_MAX))
string = QuotedString('"', escChar='\\')\
    .setName("string")\
    .addParseAction(lengthCheck())
boolean = oneOf("True False").setName("bool")\
    .setParseAction(cvtBool)
lst_elem = real | integer | string
lst = Group(lbrack+lst_elem+ZeroOrMore(comma+lst_elem)+Optional(comma)+rbrack)\
    .addParseAction(lstToList)



identifier = ~reserved_keywords+Word(lowercase, lowercasenums+"_", min=1, max=30)


key = oneOf("default min_length max_length min max help_text label hidden odd even choices required")\
    .setResultsName("property_name")
value = (real | integer | boolean | string | lst).setResultsName("property_value")
field_property = Group(key + colon + value)



def create_parser(types="Integer Dimmension Decimal Bool Text TextArea"):
    """
    Arguments:
        types: Supported types string
    """ 
    field_type = oneOf(types)

    field = Group(identifier + colon + field_type +\
                Optional(arrow+OneOrMore(field_property))\
                    .setParseAction(propToDict)\
                    .setResultsName("property_dict"))\
            .setResultsName("field").setParseAction(fieldToParam)

    params = ZeroOrMore(field)

    return params



params = create_parser(
        types = "Integer Dimmension Decimal Bool Text TextArea")

params_file = create_parser(
        types = "Integer Dimmension Decimal Bool Text TextArea File Image")

params.enablePackrat()
params_file.enablePackrat()


def parse_fields(input_str, file_support=False):
    """
    Arguments:
        input_str (string): 
        file_support (bool): Enable support to file parameters
            File
            Image
    """ 
    if file_support:
        ast = params_file.parseString(input_str, parseAll=True)
    else:
        ast = params.parseString(input_str, parseAll=True)
    
    d = OrderedDict()
    for name, field in ast:
        d[name] = field
    return d






















from django.core.exceptions import ValidationError
from pyparsing import ParseBaseException


class ParamValidator(object):
    def __init__(self, file_support=False):
        self._file_support = file_support
        
    def __call__(self, value):

        # Imported here to avoid circular dependency
        from .parser import parse_fields
        try:
            par = parse_fields(value, self._file_support)
        except ParseBaseException as err:
            # Parser Error
            raise ValidationError(str(err))
        except ValueError as err:
            # Error while creating Param
            raise ValidationError(str(err))


class ParamFormFieldValidator(object):

    def __init__(self, param):
        self._param = param

    def __call__(self, value):
        try:
            self._param.validate(value)
        except ValueError as err:
            raise ValidationError(err)
        except TypeError as err:
            raise ValidationError(err)



from django.test import TestCase
from django.core.exceptions import ValidationError
from param_field.validators import ParamValidator

class TestParamValidator(TestCase):

    def test_validator(self):
        """Test validator without file support"""
        validator = ParamValidator()

        # Control group
        input_str = """
            activate: Bool-> default:False
            message: Text-> min_length:2 max_length:20
        """
        validator(input_str)

        # Invalid parameter type (float)
        input_str = "activate: Float-> default: 3.23"
        with self.assertRaises(ValidationError):
            validator(input_str)

        # Invalid option type (default should be a bool)
        input_str = "does_not_compute: Bool->default:32"
        with self.assertRaises(ValidationError):
            validator(input_str)

        # Invalid name
        input_str = "half name: Integer-> default:55"
        with self.assertRaises(ValidationError):
            validator(input_str)

        # files not supported
        input_str = "upload: File"
        with self.assertRaises(ValidationError):
            validator(input_str)

    def test_file_validator(self):
        """Test validator with file support"""
        validator = ParamValidator(file_support=True)

        # files supported
        input_str = """
            upload: File
            enable: Bool"""
        validator(input_str)

        # Invalid control input
        input_str = "asdfasdf: Float"
        with self.assertRaises(ValidationError):
            validator(input_str)



from decimal import Decimal

INT_MAX = 999999
INT_MIN = -999999
REAL_MAX = Decimal("999999.9999") 
REAL_MIN = Decimal("-999999.9999")
STRING_MAX_LENGTH = 300
LABEL_MAX_LENGTH = 40
HELP_TEXT_MAX_LENGTH = 200




from django import forms
from django.db import models
from django.core.exceptions import ValidationError
from decimal import Decimal
from numbers import Number
from collections import OrderedDict
import json
from .limits import *

class ParamDict(OrderedDict):
  
    def __init__(self, fields='', file_support=False, parse=True):
        """
        Arguments:
            fields (str): String containig fields definitions.
            file_support(book): 
            parse(bool): If True parse field string, else just store
                original string and return empy ParamDict
        """
        from .parser import parse_fields # Solve circular import
        super(ParamDict, self).__init__()

        self._original_source = fields

        if parse:
            f = parse_fields(fields or '', file_support)
        else:
            f = {}

        for name, field in f.items():
            self[name] = field

    def __str__(self):
        if self._original_source:
            return self._original_source
        else:
            # Generate from fields
            return '\n'.join(["{}:{}"\
                .format(name, str(param)) for name, param in self.items()])

    def form(self, *args, **kwargs):
        """Return a form containig all parameters stored in ParamDict
        Arguments:
            *args and **kwargs: Are the arguments you would normally 
            pass to a form
        """
        # Imported here to avoid circular dependency
        from .forms import ParamInputForm
        
        # If there are no defined fields returns Nones instead of
        # an empty form.
        if len(self) == 0:
            return None

        kwargs['params'] = self
        return ParamInputForm(*args, **kwargs)

    def serialize(self, request):
        """Convert each value to its representation so it can be easily stored

        Argumens:
            request (dictionary): value for each parameter.
        """
        serialized = {}
        for name, value in request.items():
            serialized[name] = self[name].serialize_value(value)
        return serialized

    def deserialize(self, request):
        """The oposite step to 'request_to_dict' convert all values from its 
        representation to its real python value, that can be later be validated.
        
        Arguments:
            request (dict): Request dict containing serialized values
        """
        deserialized = {}
        for name, value in request.items():
            deserialized[name] = self[name].deserialize_value(value)
        return deserialized

    def validate(self, request):
        """
        Validate request against ParamDict parameters.

        Arguments:
            request (dict):
        """
        # Check no unknown parameter present in the request
        for name, value in request.items():
            if self.get(name, None) is None:
                raise ValidationError("Parameter '{}' does not exists"\
                        .format(name))

        # Validate request against parameter definitions
        for name, value in self.items():
            # TODO if no value was provided if there is a default value the 
            # request is valid
            try:
                value = request.get(name, None)
                param = self[name]
                if value is None:
                    default = param.get_default()
                    if default is None:
                        raise ValidationError("No value supplied for {}"\
                                .format(name))
                else:
                    self[name].validate(value)
            except (TypeError, ValueError, ValidationError) as err:
                raise ValidationError(str(err))

    def add_defaults(self, request):
        """
        Add missing parameters using default values.

        Arguments:
            request (dict): dict containing parameter values

        Returns:
            dict: new dictionary with default values
        """
        with_defaults = dict(request)

        for name, param in self.items():
            default = param.get_default()
            if default is not None and request.get(name, None) is None:
                with_defaults[name] = default

        return with_defaults

    def param_count(self):
        """Returns number of parameters in the dictionary, constants
        are not counted.
        
        Returns:
            int
        """
        params = 0
        for name, value in self.items():
            if isinstance(value, Param):
                params += 1
        return params

# Property -> allowed types | limits

class Param(object):
    native_type = str
   
    # Property and type supported, in order of initialization
    allowed_properties = [
        ('label', str, ''),
        ('help_text', str, ''),
        ('required', bool, True),
        ('choices', list, None),
        ('default', str, '')] 
    
    def _init_choices(self, value):
        """Initialize Param for choices property"""
        choices =[]
        for elem in value:
            try:
                ep = self.deserialize_value(elem)
                self.validate(ep)
                choices.append(ep)
            except (TypeError, ValueError, ValidationError):
                raise ValueError("Invalid 'choices' value '{}'".format(elem))

        self.choices = choices
    
    def _init_default(self, value):
        """Initialize Param for default property"""
        try:
            self.default = self.deserialize_value(value)
            self.validate(self.default)
        except (TypeError, ValueError, ValidationError):
            raise ValueError("Invalid 'default' value '{}'".format(value))

    def _init_label(self, value):
        """Initialize Param for label property"""
        if len(value)>LABEL_MAX_LENGTH:
            raise ValueError("'{}' label too long".format(value))
        self.label = value

    def _init_hidden(self, value):
        """Verify default value is provided when field hidden"""
        self.hidden = value
      
        # No choices are needed when the user can't seet the field
        if self.hidden and hasattr(self, 'choices'):
            self.choices = None

        # Default value required for hidden fields
        if self.hidden and (not hasattr(self, 'default') or self.default is None):
            raise ValueError('A default value must be provided for hidden parameters')

    def _init_help_text(self, value):
        if len(value)>HELP_TEXT_MAX_LENGTH:
            raise ValueError("help text too long".format(value))
        self.help_text = value

    def __init_property(self, name, value): 
        """
        Initialize property by calling its custom initialzation function or
        storing its value if none is available.
        """
        allowed_type = self._prop_type_dict[name]
        if not isinstance(value, allowed_type):
            allowed_type.__class__.__name__
            err = "'{}' expected '{}' received '{}'"\
                .format(name, allowed_type.__name__, value.__class__.__name__)
            raise ValueError(err)

        init_func = getattr(self, '_init_'+name, None)
        if init_func:
            init_func(value)
        else:
            setattr(self, name, value)
    
    def __init__(self, *args, **kwargs):
        """Custom init method responsible of initializing and checking parameters"""
        # Initialize all possible properties to default values
        self._prop_type_dict = {}
        for prop, typ, default in self.allowed_properties:
            self._prop_type_dict[prop] = typ
            setattr(self, prop, default)

        # Check only allowed properties were provided
        for prop, value in kwargs.items():
            if prop not in self._prop_type_dict:
                raise ValueError("Unexpected property '{}'".format(prop))

        # If available call custom initialization function for each property 
        # (in the order specified by allowed_properties)
        for name, typ, default in self.allowed_properties:
            if name in kwargs:
                self.__init_property(name, kwargs[name]) 

    def _validate_choices(self, value):
        if self.choices and not value in self.choices:
            raise ValidationError("Not a valid choice")

    def get_choices(self):
        """Generate a (choice, choice_str) tuple list used by form fields.
        
        Returns:
            List
            None
        """
        if hasattr(self, 'choices') and self.choices is not None:
            return [(c, str(c)) for c in self.choices]
        else:
            return None

    def get_default(self):
        if hasattr(self, 'default'):
            return self.default
        else:
            return None

    def validate(self, value):
        """Check value against parameter limits"""
        if not type(value) == self.native_type:
            err = "Expected '{}' received '{}'"\
                .format(self.native_type.__name__, type(value).__name__)
            raise TypeError(err)
        
        # Validate against available property validators
        for name, typ, default in self.allowed_properties:
            validate_func = getattr(self, '_validate_'+name, None)
            if validate_func:
                validate_func(value) 

    def is_valid(self, value):
        try:
            self.validate(value)
            return True
        except (ValueError, TypeError, ValidationError): 
            return False

    def deserialize_value(self, value):
        """Convert from value representation to the actual value object
        for example: slug string TO model instance
        USED BY FOREIGN KEY AND FILE PARAMS  
        """
        if not type(value) == self.native_type:
            err = "Expected '{}' received '{}'"\
                .format(self.native_type.__name__, type(value).__name__)
            raise TypeError(err)
        return value

    def serialize_value(self, value):
        """to_python reverse step
        USED BY FOREIGN KEY AND FILE PARAMS 
        """
        return value

    def value_to_str(self, value):
        """Convert property value to the representation used in the
        parameter definition language, so it can be parsed. This is
        later reverser by to python"""
        value = self.serialize_value(value)
        if type(value) == list:
            return '['+', '.join([self.value_to_str(v) for v in value])+']'
        elif type(value) == str:
            return json.dumps(value)
        else:
            return str(value)

    def to_str(self):
        """Convert parameter to its parameter definition language
        representation, including all properties with user defined
        values."""

        # Render properties
        prop_str = ""
        for name, typ, default in self.allowed_properties:
            
            try:
                value = getattr(self, name)
            except AttributeError:
                continue

            if value == default:
                continue
            
            if type(value)!=typ:
                err="'{}' type should be {} not {}"\
                    .format(name, str(typ), str(type(value)))
                raise ValueError(err)

            prop_str += ' {}:{}'.format(name, self.value_to_str(value))

        return self.type_name+('->'+prop_str if prop_str else '')

    def __str__(self):
         return self.to_str()


class NumberMixin(object):

    def _init_min(self, value):
        if not self.is_valid(self.native_type(value)):
            raise ValueError("Invalid 'min' value")
        self.min = value

    def _init_max(self, value):
        if not self.is_valid(self.native_type(value)):
            raise ValueError("Invalid 'max' value")
        self.max = value

    def _validate_min(self, value):
        if self.min is not None and value < self.min:
            err = "Value must be greater than or equal to {}"\
                    .format(self.min)
            raise ValidationError(err)

    def _validate_max(self, value):
        if self.max is not None and value > self.max:
            err = "Value must be smaller than or equal to {}"\
                    .format(self.max)
            raise ValidationError(err)

    def _validate_even(self, value):
        if self.even and value%2==1:
            raise ValidationError("Value must be even")

    def _validate_odd(self, value):
        if self.odd and value%2==0:
             raise ValidationError("Value must be odd")

class StringMixin(object):

    def _init_max_length(self, value):
        if value > STRING_MAX_LENGTH or value < 0:
            raise ValueError("Invalid 'max_length' value")
        self.max_length = value
    
    def _init_min_length(self, value):
        if value > STRING_MAX_LENGTH or value < 0:
            raise ValueError("Invalid 'min_length' value")
        self.min_length = value
    
    def _validate_max_length(self, value):
        if self.max_length is not None and len(value)>self.max_length:
            err = "Must be at most {} characters long".format(self.max_length)
            raise ValidationError(err)
    
    def _validate_min_length(self, value):
        if self.min_length is not None and len(value)<self.min_length:
            err = "Must be at least {} characters long".format(self.min_length)
            raise ValidationError(err)



class BoolParam(Param):
    native_type = bool
    type_name = 'Bool'
    allowed_properties = [
        ('label', str, ''),
        ('help_text', str, ''),
        ('required', bool, True),
        ('default', bool, None),
        ('hidden', bool, False)]

class IntegerParam(Param, NumberMixin):
    native_type = int
    type_name = 'Integer'
    allowed_properties = [
        ('label', str, ''),
        ('help_text', str, ''),
        ('required', bool, True),
        ('even', bool, False),
        ('odd', bool, False),
        ('min', int, INT_MIN),
        ('max', int, INT_MAX),
        ('choices', list, None),
        ('default', int, None),
        ('hidden', bool, False)]

class DecimalParam(Param, NumberMixin):
    native_type = Decimal
    type_name = 'Decimal'
    allowed_properties = [
        ('label', str, ''),
        ('help_text', str, ''),
        ('required', bool, True),
        ('min', Decimal, REAL_MIN),
        ('max', Decimal, REAL_MAX),
        ('choices', list, None),
        ('default', Decimal, None),
        ('hidden', bool, False)]

class DimmensionParam(Param, NumberMixin):
    native_type = Decimal
    type_name = 'Dimmension'
    allowed_properties = [
        ('label', str, ''),
        ('help_text', str, ''),
        ('required', bool, True),
        ('min', Decimal, Decimal('0.0')),
        ('max', Decimal, REAL_MAX),
        ('choices', list, None),
        ('default', Decimal, None),
        ('hidden', bool, False)]

class TextParam(Param, StringMixin):
    native_type = str
    type_name = 'Text'
    allowed_properties = [
        ('label', str, ''),
        ('help_text', str, ''),
        ('required', bool, True),
        ('min_length', int, 0),
        ('max_length', int, STRING_MAX_LENGTH),
        ('choices', list, None),
        ('default', str, None),
        ('hidden', bool, False)]

class TextAreaParam(TextParam):
    type_name = 'TextArea'
    pass
    #TODO: Custom widget


# File Params
class FileParam(Param, StringMixin):
    native_type = str
    type_name = 'File'
    allowed_properties = [
        ('label', str, ''),
        ('help_text', str, ''),
        ('required', bool, True)]

class ImageParam(FileParam):
    native_type = str
    type_name = 'Image'
    allowed_properties = [
        ('label', str, ''),
        ('help_text', str, ''),
        ('required', bool, True)]





from django.utils.translation import ugettext, ugettext_lazy as _
from django.core.exceptions import ValidationError
from django.db import models
from pyparsing import ParseBaseException
from .params import ParamDict
from .validators import ParamValidator


MAX_PARAM_FIELD_LENGTH = 3000

# Create your models here.

class ParamField(models.TextField):

    description = _('Parameter field')

    def __init__(self, *args, **kwargs):
        """
        Arguments:
            file_support(bool): Enable or disable support for file fields.
        """
        kwargs['max_length'] = MAX_PARAM_FIELD_LENGTH
        kwargs['blank'] = True
        
        self._file_support = kwargs.pop('file_support', True)
    
        super(ParamField, self).__init__(*args, **kwargs)

    def deconstruct(self):
        name, path, args, kwargs = super().deconstruct()
        del kwargs['max_length']
        del kwargs['blank']
        return name, path, args, kwargs

    def from_db_value(self, value, expression, connection, context):
        if value is None:
            return value

        try:
            return ParamDict(value, self._file_support)
        except ParseBaseException as err:
            # Couldn't parse form definition return empty one
            return ParamDict(value, self._file_support, parse=False)
        except ValueError as err:
            # Couldn't parse form definition return empty one
            return ParamDict(value, self._file_support, parse=False)

    def get_prep_value(self, value):
        """Convert objects to string"""
        return str(value)

    def to_python(self, value):
        if isinstance(value, ParamDict):
            return value

        if value is None:
            return Value

        try:
            return ParamDict(value, self._file_support)
        except ParseBaseException as err:
            raise ValidationError(str(err))
        except ValueError as err:
            raise ValidationError(str(err))

    def formfield(self, **kwargs):
        """So validation can be added"""
        defaults = {'validators': [ParamValidator(self._file_support),]}
        defaults.update(kwargs)
        return super(ParamField, self).formfield(**defaults)




from django import forms
from .validators import *
from .params import *


FORM_FIELD_CLASS = {
    BoolParam: forms.BooleanField,
    TextParam: forms.CharField,
    TextAreaParam: forms.CharField,
    IntegerParam: forms.IntegerField,
    DecimalParam: forms.DecimalField,
    DimmensionParam: forms.DecimalField,
    FileParam: forms.FileField,
    ImageParam: forms.ImageField,
}


def expand_name(name):
    """Convert param  name string from 'this_is_a_name' to 'This is a name' so
    it can be used as the a field label
    
    Arguments:
        -name (string):
    Returns:
        string
    """
    chars = '*+-_=,;.:\\/[]{}()<>#@$%&'
    text = name.lower()
    for c in chars:
        text = text.replace(c, ' ')
    text = text[0].upper() + text[1:]
    return text



def FileFieldFactory(param, name):
    """Form field factory for all File fields"""    
    field_args = {
        'help_text': param.help_text or None,
        'label': param.label or expand_name(name),
        'required': param.required,}

    return FORM_FIELD_CLASS[type(param)](**field_args)


def StdFieldFactory(param, name):
    """Form field  factory for fields using standard fields, that is:
    Bool, Text, TextArea, Integer, Decimal, Dimmension"""
    field_args = {
        'help_text': param.help_text or None,
        'label': param.label or expand_name(name),
        'initial': param.default,
        'required': param.required,
        'validators': [ParamFormFieldValidator(param),]}

    # Change widget for hidden fields
    if param.hidden or False:
        field_args['widget'] = forms.HiddenInput()

    # Generate correct Field type or ChoiceField
    choices = param.get_choices()
    if choices:
        field_args['coerce'] = param.native_type
        field_args['choices'] = choices
        field_args.pop('validators', None)
        return forms.TypedChoiceField(**field_args)
    else:
        return FORM_FIELD_CLASS[type(param)](**field_args)


def ParamFieldFactory(param, name):
    field_class = FORM_FIELD_CLASS[type(param)]
    if field_class in (forms.FileField, forms.ImageField):
        return FileFieldFactory(param, name)
    else:
        return StdFieldFactory(param, name)


class ParamInputForm(forms.Form):
    """
    Form with fields for the parametes of a PartGenerator, can
    be used to validate inputs
   
    Arguments:
        - params (ParamDict): Containing form fields

    Examples: 
        https://jacobian.org/writing/dynamic-form-generation/
    """
    def __init__(self, *args, **kwargs):
        """
        """
        self._params = kwargs.pop('params')
        super(ParamInputForm, self).__init__(*args, **kwargs)
       
        # Add all fields from ParamDict to the form
        for name, param in self._params.items():
            self.fields[name] = ParamFieldFactory(param, name)



from django.test import TestCase

from pyparsing import ParseBaseException, ParseException, ParseFatalException
from decimal import Decimal
from collections import OrderedDict

from param_field.parser import parse_fields
from param_field.params import *
from param_field.limits import *

class TestParserBase(TestCase):

    def test_empty_string(self):
        p = parse_fields("")
        self.assertIsInstance(p, OrderedDict)
        self.assertEqual(len(p), 0)

        p = parse_fields("          \n")
        self.assertIsInstance(p, OrderedDict)
        self.assertEqual(len(p), 0)

    def test_param_type(self):
        
        # Test supported types
        parse_fields("width1:Dimmension")
        parse_fields("width2:Integer")
        parse_fields("width3:Decimal")
        parse_fields("width4:Text")
        parse_fields("widht5:TextArea")
        parse_fields("width6:Bool")

        parse_fields("width7:File", file_support=True)
        parse_fields("width8:Image", file_support=True)
       
        # Test file_support must be enabled for File and Image
        with self.assertRaises(ParseException):
            parse_fields("width: File")

        with self.assertRaises(ParseException):
            parse_fields("widht: Image")

        # Test unknown type
        with self.assertRaises(ParseException):
            parse_fields("width:CarNumber")
       
        # Test Type case sensitive
        with self.assertRaises(ParseException):
            parse_fields("width:dimmension")
        with self.assertRaises(ParseException):
            parse_fields("width:integer")
        with self.assertRaises(ParseException):
            parse_fields("width:decimal")
        with self.assertRaises(ParseException):
            parse_fields("width:text")
        with self.assertRaises(ParseException):
            parse_fields("width:textarea")
        with self.assertRaises(ParseException):
            parse_fields("width:bool")
        with self.assertRaises(ParseException):
            parse_fields("width:file", file_support=True)
        with self.assertRaises(ParseException):
            parse_fields("width:image", file_support=True)

    def test_correct_param_used(self):
        """Test correct param object is used for each type"""
        d = parse_fields('para: Integer')
        self.assertIsInstance(d['para'], IntegerParam)
        d = parse_fields('para: Decimal')
        self.assertIsInstance(d['para'], DecimalParam)
        d = parse_fields('para: Dimmension')
        self.assertIsInstance(d['para'], DimmensionParam)
        d = parse_fields('para: Text')
        self.assertIsInstance(d['para'], TextParam)
        d = parse_fields('para: Bool')
        self.assertIsInstance(d['para'], BoolParam)
        d = parse_fields('para: File', file_support=True)
        self.assertIsInstance(d['para'], FileParam)
        d = parse_fields('para: Image', file_support=True)
        self.assertIsInstance(d['para'], ImageParam)
 
    def test_valid_names(self):
        
        parse_fields("the_name: Bool")

        # Can't start with number or underscore
        with self.assertRaises(ParseException):
            parse_fields("1_name: Bool")
        with self.assertRaises(ParseException):
            parse_fields("_name: Bool")

        # Only lowercase allowed
        with self.assertRaises(ParseException):
            parse_fields("The_name: Bool")
        
        # spaces not allowed
        with self.assertRaises(ParseException):
            parse_fields("the name: Bool")

        # Other symbols not allowed
        for s in [".-,;{}[]!=?¿|@·$%&/()*<>'¡"]:
            with self.assertRaises(ParseException):
                parse_fields("name_{}:Bool".format(s))
            with self.assertRaises(ParseException):
                parse_fields("{}:Bool".format(s))

        # Test min and max lengths
        parse_fields("a: Bool")
        parse_fields("thirty_character_long_name1111: Bool")
        with self.assertRaises(ParseException):
            parse_fields(": Bool")
        with self.assertRaises(ParseException):
            parse_fields("more_than_thirty_characters_lon: Bool")

    def test_string_parsing(self):
        """Test string quotes format"""
        # Only double quotes allowed for strings
        parse_fields('name:Text-> default:"name"')
        with self.assertRaises(ParseException):
            parse_fields("name:Text-> default:'name'")
      
        # Test string escape character
        p=parse_fields('''name:Text-> default:"name \\"string\\""''')
        self.assertEqual(p['name'].default, 'name "string"')
    
        p=parse_fields('''name:Text-> default:"asdf\\n123\\""''')
        self.assertEqual(p['name'].default, 'asdf\n123"')

        original = TextParam(default='asdf\\\nasd\n"')
        parse = parse_fields('name:'+original.to_str())
        self.assertEqual(parse['name'].default, original.default)

        # Parse string with spaces inside
        p = parse_fields('name: Text-> default:" the default string  "')
        self.assertEqual(p['name'].default, ' the default string  ')

        # Length Limit
        p = parse_fields('name: Text-> default:"{}"'\
                .format("a"*STRING_MAX_LENGTH))

        with self.assertRaises(ParseBaseException):
            parse_fields('name: Text-> default:"{}"'\
                .format("a"*(STRING_MAX_LENGTH+1)))

    def test_boolean_parsing(self):
        """Test wich boolean strings are accepted"""
        # Control group
        p = parse_fields('enable:Bool->default:True')
        self.assertIsInstance(p['enable'].default, bool)
        self.assertEqual(p['enable'].default, True)
        p = parse_fields('enable:Bool->default:False')
        self.assertIsInstance(p['enable'].default, bool)
        self.assertEqual(p['enable'].default, False)

        # Invalid boolean values
        with self.assertRaises(ParseException):
            parse_fields('enable:Bool->default:false')
        with self.assertRaises(ParseException):
            parse_fields('enable:Bool->default:true')
        with self.assertRaises(ParseException):
            parse_fields('enable:Bool->default:f')
        with self.assertRaises(ParseException):
            parse_fields('enable:Bool->default:t')
        with self.assertRaises(ParseException):
            parse_fields('enable:Bool->default:#t')
        with self.assertRaises(ParseException):
            parse_fields('enable:Bool->default:#f')
        with self.assertRaises(ValueError):
            parse_fields('enable:Bool->default:1')
        with self.assertRaises(ValueError):
            parse_fields('enable:Bool->default:0')
        with self.assertRaises(ValueError):
            parse_fields('enable:Bool->default:"True"')
        with self.assertRaises(ValueError):
            parse_fields('enable:Bool->default:"False"')
        with self.assertRaises(ValueError):
            parse_fields('enable:Bool->default:33.5')

    def test_integer_parsing(self):
        """Test valid integer values"""
        # Control group
        p = parse_fields('holes: Integer->default:0')
        self.assertEqual(p['holes'].default, 0)

        p = parse_fields('holes: Integer->default:-0')
        self.assertEqual(p['holes'].default, 0)

        p = parse_fields('holes: Integer->default:-0000001')
        self.assertEqual(p['holes'].default, -1)

        p = parse_fields('holes: Integer->default:2')
        self.assertEqual(p['holes'].default, 2)
        
        p = parse_fields('holes: Integer->default:-1')
        self.assertEqual(p['holes'].default, -1)

        # max/min limits
        p = parse_fields('holes: Integer->default:999999')
        self.assertEqual(p['holes'].default, 999999)

        p = parse_fields('holes: Integer->default:-999999')
        self.assertEqual(p['holes'].default, -999999)

        with self.assertRaises(ParseBaseException):
            parse_fields('holes: Integer->default:1000000')
        
        with self.assertRaises(ParseBaseException):
            parse_fields('holes: Integer->default:-1000000')
        
    
    def test_decimal_parsing(self):
        """Test valid decimal values"""
        # Control group
        p = parse_fields('width: Decimal->default:0.0')
        self.assertEqual(p['width'].default, Decimal('0.0'))
        self.assertEqual(p['width'].default, 0)

        p = parse_fields('width: Decimal->default:-0.0')
        self.assertEqual(p['width'].default, Decimal('0.0'))
        self.assertEqual(p['width'].default, 0)
 
        p = parse_fields('width: Decimal->default:-0000000001.0')
        self.assertEqual(p['width'].default, Decimal('-1.0'))
        self.assertEqual(p['width'].default, -1)

        # Not decimal
        with self.assertRaises(Exception):
            parse_fields('width: Decimal->default:33,4')
        
        with self.assertRaises(Exception):
            parse_fields('width: Decimal->default:33e2')

        with self.assertRaises(Exception):
            parse_fields('width: Decimal->default:33')
        
        # max/min limits   
        p = parse_fields('width: Decimal->default:999999.9999')
        self.assertEqual(p['width'].default, Decimal('999999.9999'))

        p = parse_fields('width: Decimal->default:-999999.9999')
        self.assertEqual(p['width'].default, Decimal('-999999.9999'))

        with self.assertRaises(ParseBaseException):
            parse_fields('width: Decimal->default:1000000.0')
        
        with self.assertRaises(ParseBaseException):
            parse_fields('width: Decimal->default:-1000000.0')
      

    def test_empty_choices_list(self):
        """Choices list must contain at least one element"""
        parse_fields("name:Integer-> choices:[1]")
        with self.assertRaises(ParseException):
            parse_fields("name:Integer-> choices:[]") 

    def test_errors_second_line(self):
        """Test parse errors"""
        with self.assertRaises(ParseException):
            parse_fields("number:Integer VALID:Bool")
        with self.assertRaises(ParseException):
            parse_fields("number:Integer-> default:12 hidden:Bool")
        with self.assertRaises(ParseException):
            parse_fields("number:Integer a:Bool hidden:33")
        with self.assertRaises(ParseException):
            parse_fields("number:Integer a:Bool 2:DecimalParam")
    
    def test_allowed_properties(self):
    
        # Test valid properties with correct parameter and value type.
        parse_fields('number:Integer->max:12 min:1 default:5 required:True hidden:True')
        parse_fields('number:Integer->even:True odd:True required:False hidden:False')
        parse_fields('number:Integer->default:33 choices:[22, 33, 44] required:False hidden:True')
        parse_fields('number:Text->max_length:44 min_length:1 required:False')
        parse_fields('activate:Bool->default:False hidden:False required:True')
        parse_fields('name:Text->help_text:"adfasd" label:"text input" required:False')

        # Test properties case sensitive.
        with self.assertRaises(ParseException):
            parse_fields("number:Integer-> mAx:12")
        with self.assertRaises(ParseException):
            parse_fields("input:Text-> max_length:14 Min_length:2")

        # Test properties not supported by parameter
        with self.assertRaises(ValueError):
            parse_fields("number:Integer-> max_length:12")
        with self.assertRaises(ValueError):
            parse_fields("number:Integer-> min_length:12")
       
        with self.assertRaises(ValueError):
            parse_fields("number:Decimal-> odd:True")
        with self.assertRaises(ValueError):
            parse_fields("number:Decimal-> even:True")
        with self.assertRaises(ValueError):
            parse_fields("number:Decimal-> max_length:12")
        with self.assertRaises(ValueError):
            parse_fields("number:Decimal-> min_length:12")
       
        with self.assertRaises(ValueError):
            parse_fields("number:Dimmension-> odd:True")
        with self.assertRaises(ValueError):
            parse_fields("number:Dimmension-> even:True")
        with self.assertRaises(ValueError):
            parse_fields("number:Dimmension-> max_length:12")
        with self.assertRaises(ValueError):
            parse_fields("number:Dimmension-> min_length:12")

        with self.assertRaises(ValueError):
            parse_fields("comment:Text-> odd:True")
        with self.assertRaises(ValueError):
            parse_fields("comment:Text-> even:True")
        with self.assertRaises(ValueError):
            parse_fields("comment:Text-> min:True")
        with self.assertRaises(ValueError):
            parse_fields("comment:Text-> max:True")

        with self.assertRaises(ValueError):
            parse_fields("activate:Bool-> odd:True")
        with self.assertRaises(ValueError):
            parse_fields("activate:Bool-> even:True")
        with self.assertRaises(ValueError):
            parse_fields("activate:Bool-> min:1")
        with self.assertRaises(ValueError):
            parse_fields("activate:Bool-> max:2")
        with self.assertRaises(ValueError):
            parse_fields("activate:Bool-> min_length:1")
        with self.assertRaises(ValueError):
            parse_fields("activate:Bool-> max_length:3")
   
        # Test invalid properties
        with self.assertRaises(ParseException):
            parse_fields("activate:Bool-> purse:3")
        with self.assertRaises(ParseException):
            parse_fields("number:Decimal-> car:3")

        # Test no properties are required
        parse_fields("activate:Bool")

    def test_properties_initialized(self):
        """Test properties are passed to param __init__ and initialized"""
        p = parse_fields('number:Integer->max:12 min:1 default:5 hidden:True')
        self.assertEqual(p['number'].max, 12)
        self.assertEqual(p['number'].min, 1)
        self.assertEqual(p['number'].default, 5)
        self.assertEqual(p['number'].hidden, True)

        p = parse_fields('number:Integer->even:True odd:True required:False')
        self.assertEqual(p['number'].even, True)
        self.assertEqual(p['number'].odd, True)
        self.assertEqual(p['number'].required, False)

        p = parse_fields('number:Integer->choices:[3, 4, 5, 6]')
        self.assertEqual(p['number'].choices[0], 3)
        self.assertEqual(p['number'].choices[1], 4)
        self.assertEqual(p['number'].choices[2], 5)
        self.assertEqual(p['number'].choices[3], 6)

        p = parse_fields('name:Text-> min_length:1 max_length:4')
        self.assertEqual(p['name'].min_length, 1)
        self.assertEqual(p['name'].max_length, 4)

        p = parse_fields('name:Text-> default:"324"')
        self.assertEqual(p['name'].default, '324')

        p = parse_fields('active:Bool-> label:"the real" help_text:"helpy"')
        self.assertEqual(p['active'].label, "the real")
        self.assertEqual(p['active'].help_text, "helpy")

        p = parse_fields('width:Dimmension-> default:12.1 hidden:True')
        self.assertEqual(p['width'].hidden, True)
        self.assertEqual(p['width'].default, Decimal('12.1'))

        p = parse_fields('custom_file:File-> required:False', file_support=True)
        self.assertEqual(p['custom_file'].required, False)

    def test_properties_defaults(self):
        """Test properties default values"""
        p = parse_fields('par:Bool')
        self.assertEqual(p['par'].label, '')
        self.assertEqual(p['par'].help_text, '')
        self.assertEqual(p['par'].default, None)
        self.assertEqual(p['par'].hidden, False)
        self.assertEqual(p['par'].required, True)

        p = parse_fields('par:Text')
        self.assertEqual(p['par'].label, '')
        self.assertEqual(p['par'].help_text, '')
        self.assertEqual(p['par'].default, None)
        self.assertEqual(p['par'].choices, None)
        self.assertEqual(p['par'].max_length, STRING_MAX_LENGTH)
        self.assertEqual(p['par'].min_length, 0)
        self.assertEqual(p['par'].hidden, False)
        self.assertEqual(p['par'].required, True)

        p = parse_fields('par:Integer')
        self.assertEqual(p['par'].label, '')
        self.assertEqual(p['par'].help_text, '')
        self.assertEqual(p['par'].default, None)
        self.assertEqual(p['par'].choices, None)
        self.assertEqual(p['par'].max, INT_MAX)
        self.assertEqual(p['par'].min, INT_MIN)
        self.assertEqual(p['par'].even, False)
        self.assertEqual(p['par'].odd, False) 
        self.assertEqual(p['par'].hidden, False)
        self.assertEqual(p['par'].required, True)

        p = parse_fields('par:Decimal')
        self.assertEqual(p['par'].label, '')
        self.assertEqual(p['par'].help_text, '')
        self.assertEqual(p['par'].default, None)
        self.assertEqual(p['par'].choices, None)
        self.assertEqual(p['par'].max, REAL_MAX)
        self.assertEqual(p['par'].min, REAL_MIN)  
        self.assertEqual(p['par'].hidden, False)
        self.assertEqual(p['par'].required, True)

        p = parse_fields('par:Dimmension')
        self.assertEqual(p['par'].label, '')
        self.assertEqual(p['par'].help_text, '')
        self.assertEqual(p['par'].default, None)
        self.assertEqual(p['par'].choices, None)
        self.assertEqual(p['par'].max, REAL_MAX)
        self.assertEqual(p['par'].min, Decimal('0.0'))
        self.assertEqual(p['par'].hidden, False)
        self.assertEqual(p['par'].required, True)
        
        p = parse_fields('par:File', file_support=True)
        self.assertEqual(p['par'].label, '')
        self.assertEqual(p['par'].help_text, '')
        self.assertEqual(p['par'].required, True)

        p = parse_fields('par:Image', file_support=True)
        self.assertEqual(p['par'].label, '')
        self.assertEqual(p['par'].help_text, '')
        self.assertEqual(p['par'].required, True)



from django.test import TestCase
from django.core.exceptions import ValidationError
from unittest.mock import patch
from param_field.params import *
from param_field.forms import ParamInputForm

from param_field.limits import (INT_MAX, INT_MIN, REAL_MAX, REAL_MIN,
        STRING_MAX_LENGTH, LABEL_MAX_LENGTH, HELP_TEXT_MAX_LENGTH)


from decimal import Decimal

class TestParamDict(TestCase):

    def test_param_count(self): 
        """Test param_count returns correct number"""
        d = ParamDict("""
            bool:Bool->default:True label:"enable"
            int:Integer->default:44 label:"number of holes" """)
        self.assertEqual(d.param_count(), 2)
 
        d = ParamDict(""" """)
        self.assertEqual(d.param_count(), 0)

        #TODO:  Test dict with contants when implemented

    def test_validate(self):
        p1 = BoolParam(default=True, label="enable")
        p2 = IntegerParam(default=44, label="number of holes")
        d = ParamDict("""
            bool: Bool-> default:True label:"enable"
            int: Integer-> default:44 label:"number of holes" """)
      
        # Control group
        d.validate({'bool': True, 'int': 12})

        # Test fields with a default value are not required
        d.validate({'bool': False})
        d.validate({'int': 34})
        d.validate({})
        
        # Test invalid values raise Validation exception
        with self.assertRaises(ValidationError):
            d.validate({'bool': 12})
        with self.assertRaises(ValidationError):
            d.validate({'bool': 'True'})
        with self.assertRaises(ValidationError):
            d.validate({'int': True})
        with self.assertRaises(ValidationError):
            d.validate({'int': Decimal('12')})

        # Test against params with some restriction
        d = ParamDict("""
            int_odd:Integer->odd:True
            int:Integer->default:44 max:55 label:"number of holes" """)

        d.validate({'int':55, 'int_odd': 45})
        d.validate({'int_odd': 1})

        with self.assertRaises(ValidationError):
            d.validate({'int_odd': 4})

        with self.assertRaises(ValidationError):
            d.validate({'int_odd': 1, 'int': 56})

        with self.assertRaises(ValidationError):
            d.validate({})

    def test_add_defaults(self):
        """Test add_defaults method"""
        d = ParamDict("""
            bool:Bool->default:True label:"enable"
            int:Integer->default:44 label:"number of holes" """)
      
        with_d = d.add_defaults({})
        self.assertEqual(with_d['bool'], True)
        self.assertEqual(with_d['int'], 44)
        
        with_d = d.add_defaults({'int': 33, 'bool':False})
        self.assertEqual(with_d['bool'], False)
        self.assertEqual(with_d['int'], 33)
        
        d = ParamDict("""
            int_odd:Integer-> odd:True
            int:Integer-> default:44""")

        with_d = d.add_defaults({})
        self.assertEqual(with_d['int'], 44)
        self.assertTrue(len(with_d), 1)
        
    def test_serialize(self):
        # TODO: Test when file fields are implemented
        pass

    def test_deserialize(self):
        # TODO: Test when file fields are implemented
        pass

    def test_string_representation(self):
        
        # Empty dict mapped into empty string
        d = ParamDict()
        self.assertEqual(str(d), '')

        # Check original source preserved.
        field_def = """
            name:Bool-> default:True label:"the label" """
        d = ParamDict(field_def)

        self.assertEqual(str(d), field_def)
  
        # Two parameters check order preserved
        d = ParamDict("""
            bool:Bool->default:True label:"enable"
            int:Integer->default:44 label:"number of holes" """)

        self.assertTrue('Bool' in str(d))
        self.assertTrue('enable' in str(d))
        self.assertTrue('True' in str(d))
        self.assertTrue('Integer' in str(d))
       
        # Test rest of param types
        d = ParamDict("""
            p1: Decimal
            p2: Dimmension
            p3: Text
            p4: Text""")

        for name in ('p1', 'p2', 'p3', 'p4'):
            self.assertTrue(name in str(d))

    def test_form_generation(self): 

        # test ParamDict.form() generates a valid Form
        d = ParamDict("""
            name_12345:Bool-> default:True label:"the label" """)
        form = d.form()

        self.assertIsInstance(form, ParamInputForm)
        self.assertTrue('name_12345' in form.as_p())

        
        # Test empty Dict returns None instead of empty form
        d = ParamDict("")
        form = d.form()
        self.assertEqual(form, None)



class TestBaseParam(TestCase):


    def test_unknown_properties(self):
        """Test error raised when unsuported properties provided
        during creation"""
        param = TextParam(label='the_param', max_length=20)

        with self.assertRaises(ValueError):
            param=TextParam(label='the name', invented_param=43)

    def test_property_type(self):
        """Test properties type are checked"""

        # Common params
        TextParam(label='a label')
        with self.assertRaises(ValueError):
            TextParam(label=12)

        TextParam(help_text='some boring explanation')
        with self.assertRaises(ValueError):
            TextParam(help_text=12)

        IntegerParam(default=12)
        DecimalParam(default=Decimal('12'))
        DimmensionParam(default=Decimal('14'))
        TextParam(default='asdf')
        TextAreaParam(default='3')
        with self.assertRaises(ValueError):
            IntegerParam(default='12')
        with self.assertRaises(ValueError):
            DecimalParam(default=12)
        with self.assertRaises(ValueError):
            DimmensionParam(default=14)
        with self.assertRaises(ValueError):
            TextParam(default=2)
        with self.assertRaises(ValueError):
            TextAreaParam(default=3)
        
        DecimalParam(choices=[Decimal('34.2'), Decimal('44.0')])
        with self.assertRaises(ValueError):
            DecimalParam(choices=[23, Decimal('55.3')])
        with self.assertRaises(ValueError):
            DecimalParam(choices=["44.5"])
        with self.assertRaises(ValueError):
            DecimalParam(choices=Decimal("44.5"))

        IntegerParam(label="3434")
        with self.assertRaises(ValueError):
            IntegerParam(label=23)
        
        IntegerParam(help_text="some helpy text")
        with self.assertRaises(ValueError):
            IntegerParam(help_text=12)

        IntegerParam(hidden=False, default=22)
        with self.assertRaises(ValueError):
            IntegerParam(hidden=0)

        # String params
        TextParam(max_length=32)
        with self.assertRaises(ValueError):
            TextParam(max_length="32")

        TextAreaParam(min_length=3)
        with self.assertRaises(ValueError):
            TextAreaParam(min_length="3")
        
        # Numeric params
        DecimalParam(max=Decimal("43.4"))
        IntegerParam(max=23)
        with self.assertRaises(ValueError):
            DecimalParam(max='42')
        with self.assertRaises(ValueError):
            IntegerParam(max='2')

        # Integer params
        IntegerParam(odd=True)
        IntegerParam(even=True)
        with self.assertRaises(ValueError):
            IntegerParam(odd=12)
        with self.assertRaises(ValueError):
            IntegerParam(even="12")

    def test_property_limits(self):
        """Test properties limit values default, choices"""
        TextParam(label="a"*LABEL_MAX_LENGTH)
        with self.assertRaises(ValueError):
            TextParam(label="b"*(LABEL_MAX_LENGTH+1))

        TextParam(help_text="b"*HELP_TEXT_MAX_LENGTH)
        with self.assertRaises(ValueError):
            TextParam(help_text="c"*(HELP_TEXT_MAX_LENGTH+1))

        IntegerParam(default=INT_MAX)
        with self.assertRaises(ValueError):
            IntegerParam(default=INT_MAX+1)
        IntegerParam(default=INT_MIN)
        with self.assertRaises(ValueError):
            IntegerParam(default=INT_MIN-1)

        DecimalParam(choices=[Decimal('12'), REAL_MAX])
        with self.assertRaises(ValueError):
            IntegerParam(choices=[Decimal('12'), REAL_MAX+1])

        DecimalParam(choices=[Decimal('12'), REAL_MIN])
        with self.assertRaises(ValueError):
            IntegerParam(choices=[Decimal('12'), REAL_MIN-1])

        TextParam(default="a"*STRING_MAX_LENGTH)
        with self.assertRaises(ValueError):
            TextParam(default="b"*(STRING_MAX_LENGTH+1))

    def test_get_choices(self):
        """Test returns choice-name pair list"""
        p = IntegerParam(choices=[12, 34])
        self.assertEqual(p.get_choices(), [(12, '12'), (34, '34')])

    def test_restrinctions(self):
        TextParam(default="asdf", choices=["asdf", "1234"])
        with self.assertRaises(ValueError):
            TextParam(default="asdf", choices=["1234", "4567"])
        
        IntegerParam(default=13, choices=[12, 13, 14])
        with self.assertRaises(ValueError):
            TextParam(default=13, choices=[12, 14])

        DecimalParam(default=Decimal('12'), choices=[Decimal('12')])
        with self.assertRaises(ValueError):
            DecimalParam(default=Decimal('12'), choices=[Decimal('13')])
        with self.assertRaises(ValueError):
            DecimalParam(default=13, choices=[Decimal('13')])

        # Test label and help_text max_length
        DecimalParam(label='a'*LABEL_MAX_LENGTH)
        DecimalParam(help_text='b'*HELP_TEXT_MAX_LENGTH)
        with self.assertRaises(ValueError):
            DecimalParam(label='a'*(LABEL_MAX_LENGTH+1))
        with self.assertRaises(ValueError):
            DecimalParam(help_text='b'*(HELP_TEXT_MAX_LENGTH+1))

    def test_label_help(self):
        """Test label and help_text"""
        b = BoolParam()
        self.assertEqual(b.label, '')
        self.assertEqual(b.help_text, '')

        b = BoolParam(label="asd", help_text="help")
        self.assertEqual(b.label, 'asd')
        self.assertEqual(b.help_text, 'help')

    def test_choices_empty_list(self):
        """Test choices list must contain at least one element"""
        b = IntegerParam(choices=[])

    def test_to_str(self):
        """ """
        self.assertEqual(BoolParam().to_str(), 'Bool')
        self.assertEqual(IntegerParam().to_str(), 'Integer')
        self.assertEqual(DecimalParam().to_str(), 'Decimal')
        self.assertEqual(DimmensionParam().to_str(), 'Dimmension')
        self.assertEqual(TextParam().to_str(), 'Text')

        p = BoolParam(label='the name')
        self.assertEqual(p.to_str(), 'Bool-> label:"the name"')

        p = BoolParam(help_text='the helpy help')
        self.assertEqual(p.to_str(), 'Bool-> help_text:"the helpy help"')

        p = BoolParam(default=False)
        self.assertEqual(p.to_str(), 'Bool-> default:False')
        
        p = BoolParam(default=True)
        self.assertEqual(p.to_str(), 'Bool-> default:True')

        p = BoolParam(default=True, hidden=False)
        self.assertEqual(p.to_str(), 'Bool-> default:True')

        p = DecimalParam(min=Decimal('12.0'), max=Decimal('20.4'))
        self.assertEqual(p.to_str(), 'Decimal-> min:12.0 max:20.4')

        p = TextParam(max_length=20, min_length=1)
        self.assertEqual(p.to_str(), 'Text-> min_length:1 max_length:20')

        p = DimmensionParam(default=Decimal('12.2'))
        self.assertEqual(p.to_str(), 'Dimmension-> default:12.2')

        p = IntegerParam(even=True, odd=True)
        self.assertEqual(p.to_str(), 'Integer-> even:True odd:True')

        #Choices
        p = IntegerParam(choices=[1, 2, 3])
        self.assertEqual(p.to_str(), 'Integer-> choices:[1, 2, 3]')

        p = TextParam(choices=["a", "b", "c"])
        self.assertEqual(p.to_str(), 'Text-> choices:["a", "b", "c"]')

        p = DecimalParam(choices=[Decimal("1.2"), Decimal("2.5")])
        self.assertEqual(p.to_str(), 'Decimal-> choices:[1.2, 2.5]')

        p = DimmensionParam(choices=[Decimal("0.0")])
        self.assertEqual(p.to_str(), 'Dimmension-> choices:[0.0]')

        # String escaping
        p = TextParam(default='asdf\n123"')
        self.assertEqual(p.to_str(), '''Text-> default:"asdf\\n123\\""''')

    def test_is_valid(self):
        """Just test it calls validate method, more test with each
        param subclass"""
        with patch.object(Param, 'validate') as mockmeth:
            p = IntegerParam()

            # Validation Error
            mockmeth.side_effect = ValidationError('Invalid value')
            self.assertFalse(p.is_valid(12))
            self.assertEqual(mockmeth.call_count, 1)

            mockmeth.side_effect = None
            self.assertTrue(p.is_valid(12))
            self.assertTrue(p.is_valid("no validation"))
            self.assertEqual(mockmeth.call_count, 3)

        with patch.object(Param, 'validate') as mockmeth:
            p = IntegerParam()
            # Type Error
            mockmeth.side_effect = TypeError('Invalid type')
            self.assertFalse(p.is_valid(12))
            self.assertEqual(mockmeth.call_count, 1)

            mockmeth.side_effect = None
            self.assertTrue(p.is_valid(12))
            self.assertTrue(p.is_valid("no validation"))
            self.assertEqual(mockmeth.call_count, 3)

        with patch.object(Param, 'validate') as mockmeth:
            p = IntegerParam()
            # Value Error 
            mockmeth.side_effect = ValueError('Invalid type')
            self.assertFalse(p.is_valid(12))
            self.assertEqual(mockmeth.call_count, 1)

            mockmeth.side_effect = None
            self.assertTrue(p.is_valid(12))
            self.assertTrue(p.is_valid("no validation"))
            self.assertEqual(mockmeth.call_count, 3)

    def test_validate(self):
        # Test checks type
        p = BoolParam()
        p.validate(True)
        with self.assertRaises(TypeError):
            p.validate(12)

        # Test call custom properties validators
        with patch.object(IntegerParam, '_validate_min') as mock_min:
            p = IntegerParam()
            self.assertEqual(mock_min.call_count, 0)
            p.validate(12)
            self.assertEqual(mock_min.call_count, 1)

            mock_min.side_effect = ValidationError('invalid value')
            with self.assertRaises(ValidationError):
                p.validate(12)

    def test_hidden(self):
        """Test hidden parameters require default value"""
        IntegerParam(hidden=True, default=12)
        with self.assertRaises(ValueError):
            IntegerParam(hidden=True)

        DecimalParam(hidden=True, default=Decimal('23.3'))
        with self.assertRaises(ValueError):
            DecimalParam(hidden=True)

        DimmensionParam(hidden=True, default=Decimal('4'))
        with self.assertRaises(ValueError):
            DimmensionParam(hidden=True)
       
        BoolParam(hidden=True, default=True)
        with self.assertRaises(ValueError):
            BoolParam(hidden=True)

        TextParam(hidden=True, default="adfasd")
        with self.assertRaises(ValueError):
            TextParam(hidden=True)

        TextAreaParam(hidden=True, default="asdf")
        with self.assertRaises(ValueError):
            TextAreaParam(hidden=True)

        # TODO: Test choices are ignored when field is hidden


class TestBoolParam(TestCase):
    
    def test_unsupported_properties(self):
        """Test properties not supported by bool"""
        with self.assertRaises(ValueError):
            BoolParam(choices=[True, False])
        with self.assertRaises(ValueError):
            BoolParam(max=12)
        with self.assertRaises(ValueError):
            BoolParam(max=True)
        with self.assertRaises(ValueError):
            BoolParam(min=13)
        with self.assertRaises(ValueError):
            BoolParam(odd=True)
        with self.assertRaises(ValueError):
            BoolParam(even=True)
        with self.assertRaises(ValueError):
            BoolParam(max_length=30)
        with self.assertRaises(ValueError):
            BoolParam(min_length=1)

    def test_is_valid(self):
        p = BoolParam(default=False)

        self.assertTrue(p.is_valid(True))
        self.assertTrue(p.is_valid(False))

        self.assertFalse(p.is_valid(None))
        self.assertFalse(p.is_valid(12))
        self.assertFalse(p.is_valid("23"))
        self.assertFalse(p.is_valid(0))
        self.assertFalse(p.is_valid(1))

        p = BoolParam()
        self.assertTrue(p.is_valid(True))
        self.assertTrue(p.is_valid(False))
        self.assertFalse(p.is_valid(None))


class TestTextParam(TestCase):

    param = TextParam
    def test_unsupported_properties(self):
        with self.assertRaises(ValueError):
            self.param(max=12)
        with self.assertRaises(ValueError):
            self.param(min=10)
        with self.assertRaises(ValueError):
            self.param(odd=True)
        with self.assertRaises(ValueError):
            self.param(even=True)

    def test_restrictions(self):
        """ """
        self.param(max_length=9, min_length=2, default="12")
        self.param(max_length=9, min_length=2, default="123456789")
        self.param(max_length=9, min_length=2, default="12",
                choices=["12", "123", "123456789"])

        with self.assertRaises(ValueError):
            self.param(max_length=9, min_length=2, default="1")
        with self.assertRaises(ValueError):
            self.param(max_length=9, min_length=2, default="1234567890")
        with self.assertRaises(ValueError):
            self.param(max_length=9, min_length=2, choices=["1", "123"])
        with self.assertRaises(ValueError):
            self.param(max_length=9, min_length=2, choices=["12", "1234567890"])
        with self.assertRaises(ValueError):
            self.param(min_length=-1)
        with self.assertRaises(ValueError):
            self.param(max_length=STRING_MAX_LENGTH+1)

    def test_is_valid(self):
        p = self.param(min_length=3,)
        self.assertTrue(p.is_valid("1234"))
        self.assertTrue(p.is_valid("123"))

        self.assertFalse(p.is_valid("12"))
        self.assertFalse(p.is_valid(12))
        self.assertFalse(p.is_valid(False))
        self.assertFalse(p.is_valid(""))
        self.assertFalse(p.is_valid(None))
        
        p = TextParam(min_length=4, default="12345")
        self.assertFalse(p.is_valid(None))



class TestTextAreaParam(TestTextParam):
    param=TextAreaParam


class TestIntegerParam(TestCase):
   
    def test_unsupported_properties(self):
        with self.assertRaises(ValueError):
            IntegerParam(max_length=12)
        with self.assertRaises(ValueError):
            IntegerParam(min_length=44)

    def test_restrictions(self):
        """Test provided parameter restrictions are applied to
        accepted values"""
        IntegerParam(default=12)
        with self.assertRaises(ValueError):
            IntegerParam(min=13, default=12)
        with self.assertRaises(ValueError):
            IntegerParam(min=13, choices=[11, 14])

        IntegerParam(max = 12, default=12)
        with self.assertRaises(ValueError):
            IntegerParam(max=11, default=12)
        with self.assertRaises(ValueError):
            IntegerParam(max=11, default=[12, 1])
  
        IntegerParam(odd=True, default=11)
        with self.assertRaises(ValueError):
            IntegerParam(odd=True, default=2)
        with self.assertRaises(ValueError):
            IntegerParam(odd=True, choices=[4])

        IntegerParam(even=True, default=12)
        with self.assertRaises(ValueError):
            IntegerParam(even=True, default=1)

        # Property combination
        with self.assertRaises(ValueError):
            IntegerParam(even=True, odd=True, default=2)
        with self.assertRaises(ValueError):
            IntegerParam(even=True, odd=True, default=1)

        IntegerParam(max=12, min=10, default=11)
        IntegerParam(max=11, min=11, default=11)
        with self.assertRaises(ValueError):
            IntegerParam(max=12, min=10, default=13)
        with self.assertRaises(ValueError):
            IntegerParam(max=12, min=10, default=9)

        IntegerParam(max=13, odd=True, default=13)
        with self.assertRaises(ValueError):
            IntegerParam(max=13, odd=True, default=15)
        with self.assertRaises(ValueError):
            IntegerParam(max=13, odd=True, default=8)
        
        # Crossing max/min values
        with self.assertRaises(ValueError):
            IntegerParam(max=12, min=13)

        # Default restrictions
        IntegerParam(default=INT_MAX)
        IntegerParam(default=INT_MIN)
        with self.assertRaises(ValueError):
            IntegerParam(default=INT_MAX+1)
        with self.assertRaises(ValueError):
            IntegerParam(default=INT_MIN-1)

    def test_is_valid(self):
        p = IntegerParam(max=12)
        self.assertTrue(p.is_valid(12))
        self.assertFalse(p.is_valid(14))

        p = IntegerParam(max=13, min=12)
        self.assertTrue(p.is_valid(13))
        self.assertTrue(p.is_valid(12))
        self.assertFalse(p.is_valid(11))
        self.assertFalse(p.is_valid(14))

        p = IntegerParam(choices=[12, 45, 33])
        self.assertTrue(p.is_valid(12))
        self.assertTrue(p.is_valid(45))
        self.assertTrue(p.is_valid(33))
        self.assertFalse(p.is_valid(13))

        p = IntegerParam(odd=True)
        self.assertTrue(p.is_valid(11))
        self.assertFalse(p.is_valid(-2))

        p = IntegerParam(even=True, max=30)
        self.assertTrue(p.is_valid(28))
        self.assertTrue(p.is_valid(30))
        self.assertFalse(p.is_valid(32))
        self.assertFalse(p.is_valid(23))
        self.assertFalse(p.is_valid(None))

        p = IntegerParam(default=12)
        self.assertFalse(p.is_valid(None))



class TestDecimalParam(TestCase):
    #TODO:
    def test_unsupported_properties(self):
        with self.assertRaises(ValueError):
            DecimalParam(max_length=12)
        with self.assertRaises(ValueError):
            DecimalParam(min_length=1)
        with self.assertRaises(ValueError):
            DecimalParam(odd=False)
        with self.assertRaises(ValueError):
            DecimalParam(even=False)

    def test_restrictions(self):
 
        DecimalParam(default=Decimal("12"))
        with self.assertRaises(ValueError):
            DecimalParam(min=Decimal("13"), default=Decimal("12"))
        with self.assertRaises(ValueError):
            DecimalParam(min=Decimal("13"), 
                    choices=[Decimal("11"), Decimal("14")])

        DecimalParam(max=Decimal("12"), default=Decimal("-12"))
        with self.assertRaises(ValueError):
            DecimalParam(max=Decimal("11"), default=Decimal("12"))
        with self.assertRaises(ValueError):
            DecimalParam(max=Decimal("11"), 
                    default=[Decimal("12"), Decimal("1")])
 
        # Choices 
        DecimalParam(max=Decimal("12"), choices=[Decimal("1"), Decimal("2")],
            default=Decimal("1"))

        with self.assertRaises(ValueError):
            DecimalParam(max=Decimal("12"), choices=[Decimal("1"), Decimal("2")],
                default=Decimal("3"))

        # Property combination
        DecimalParam(max=Decimal("12"), min=Decimal("10"), 
                default=Decimal("11"))
        DecimalParam(max=Decimal("11"), min=Decimal("11"), 
                default=Decimal("11"))
        with self.assertRaises(ValueError):
            DecimalParam(max=Decimal("12"), min=Decimal("10"), 
                    default=Decimal("13"))
        with self.assertRaises(ValueError):
            DecimalParam(max=Decimal("12"), min=Decimal("10"), 
                    default=Decimal("9"))

        # Crossing max/min values
        with self.assertRaises(ValueError):
            DecimalParam(max=Decimal("12"), min=Decimal("13"))

        # Types
        with self.assertRaises(ValueError):
            DecimalParam(max=12)
        with self.assertRaises(ValueError):
            DecimalParam(max=False)
        with self.assertRaises(ValueError):
            DecimalParam(max=None)
        with self.assertRaises(ValueError):
            DecimalParam(max="33")
 
        with self.assertRaises(ValueError):
            DecimalParam(min=12)
        with self.assertRaises(ValueError):
            DecimalParam(min=False)
        with self.assertRaises(ValueError):
            DecimalParam(min=None)
        with self.assertRaises(ValueError):
            DecimalParam(min="33")

        # Default restriction
        DecimalParam(max=REAL_MAX)
        DecimalParam(max=REAL_MIN)
        DecimalParam(min=REAL_MIN)
        DecimalParam(min=REAL_MAX)
        with self.assertRaises(ValueError):
            DecimalParam(max=REAL_MAX+1)
        with self.assertRaises(ValueError):
            DecimalParam(max=REAL_MIN-1)
        with self.assertRaises(ValueError):
            DecimalParam(min=REAL_MIN-1)
        with self.assertRaises(ValueError):
            DecimalParam(min=REAL_MAX+1)

    def test_is_valid(self):
        p = DecimalParam(max=Decimal("44"))
        self.assertTrue(p.is_valid(Decimal("12")))
        self.assertTrue(p.is_valid(Decimal("-12")))

        self.assertFalse(p.is_valid(Decimal("555")))
        self.assertFalse(p.is_valid(None))
        self.assertFalse(p.is_valid(12))
        self.assertFalse(p.is_valid("sdf"))
        self.assertFalse(p.is_valid(True))

        p = DecimalParam(max=Decimal("44"), default=Decimal("22"))
        self.assertFalse(p.is_valid(None))



class TestDimmensionParam(TestCase):
    
    def test_unsupported_properties(self):
        with self.assertRaises(ValueError):
            DimmensionParam(max_length=12)
        with self.assertRaises(ValueError):
            DimmensionParam(min_length=1)
        with self.assertRaises(ValueError):
            DimmensionParam(odd=False)
        with self.assertRaises(ValueError):
            DimmensionParam(even=False)

    def test_always_positive(self):
        p = DimmensionParam()
        self.assertTrue(p.is_valid(Decimal("0")))
        self.assertFalse(p.is_valid(Decimal("-1")))

        with self.assertRaises(ValueError):
            p = DimmensionParam(min=Decimal("-1"))
 
    def test_restrictions(self): 
        DimmensionParam(default=Decimal("12"))
        with self.assertRaises(ValueError):
            DimmensionParam(min=Decimal("13"), default=Decimal("12"))
        with self.assertRaises(ValueError):
            DimmensionParam(min=Decimal("13"), 
                    choices=[Decimal("11"), Decimal("14")])

        DimmensionParam(max=Decimal("12"), default=Decimal("1"))
        with self.assertRaises(ValueError):
            DimmensionParam(max=Decimal("11"), default=Decimal("12"))
        with self.assertRaises(ValueError):
            DimmensionParam(max=Decimal("11"), 
                    default=[Decimal("12"), Decimal("1")])
  
        # Property combination
        DimmensionParam(max=Decimal("12"), min=Decimal("10"), 
                default=Decimal("11"))
        DimmensionParam(max=Decimal("11"), min=Decimal("11"), 
                default=Decimal("11"))
        with self.assertRaises(ValueError):
            DimmensionParam(max=Decimal("12"), min=Decimal("10"), 
                    default=Decimal("13"))
        with self.assertRaises(ValueError):
            DimmensionParam(max=Decimal("12"), min=Decimal("10"), 
                    default=Decimal("9"))

        # Crossing max/min values
        with self.assertRaises(ValueError):
            DimmensionParam(max=Decimal("12"), min=Decimal("13"))

        # Types
        with self.assertRaises(ValueError):
            DimmensionParam(max=12)
        with self.assertRaises(ValueError):
            DimmensionParam(max=False)
        with self.assertRaises(ValueError):
            DimmensionParam(max=None)
        with self.assertRaises(ValueError):
            DimmensionParam(max="33")
 
        with self.assertRaises(ValueError):
            DimmensionParam(min=12)
        with self.assertRaises(ValueError):
            DimmensionParam(min=False)
        with self.assertRaises(ValueError):
            DimmensionParam(min=None)
        with self.assertRaises(ValueError):
            DimmensionParam(min="33")

        # Default restriction
        DimmensionParam(max=REAL_MAX)
        DimmensionParam(max=Decimal("0"))
        DimmensionParam(min=REAL_MAX)
        DimmensionParam(min=Decimal("0"))
        with self.assertRaises(ValueError):
            DimmensionParam(max=REAL_MAX+1)
        with self.assertRaises(ValueError):
            DimmensionParam(max=Decimal("-1"))
        with self.assertRaises(ValueError):
            DimmensionParam(min=REAL_MAX+1)
        with self.assertRaises(ValueError):
            DimmensionParam(min=Decimal("-1"))

    def test_is_valid(self):
        p = DimmensionParam(max=Decimal("44"))
        self.assertTrue(p.is_valid(Decimal("12")))
        self.assertTrue(p.is_valid(Decimal("0")))

        self.assertFalse(p.is_valid(Decimal("-1")))
        self.assertFalse(p.is_valid(Decimal("555")))
        self.assertFalse(p.is_valid(None))
        self.assertFalse(p.is_valid(12))
        self.assertFalse(p.is_valid("sdf"))
        self.assertFalse(p.is_valid(True))

        p = DimmensionParam(max=Decimal("44"), default=Decimal("22"))
        self.assertFalse(p.is_valid(None))





class TestFileParam(TestCase):    
    param = FileParam

    def test_unsupported_properties(self):
        with self.assertRaises(ValueError):
            self.param(max=12)
        with self.assertRaises(ValueError):
            self.param(min=10)
        with self.assertRaises(ValueError):
            self.param(odd=True)
        with self.assertRaises(ValueError):
            self.param(even=True)
        with self.assertRaises(ValueError):
            self.param(hidden=True)
        with self.assertRaises(ValueError):
            self.param(max_length=12)
        with self.assertRaises(ValueError):
            self.param(default=12)

    def test_is_valid(self):
        p = TextParam(required=True)
        self.assertTrue(p.is_valid("fasd"))





from setuptools import setup

setup(
	name='django-param-field',
	version='0.3',
	description='A Django model field that uses a DSL to define, generate, and validate, custom forms',
	url='https://github.com/secnot/django-param-field',
	author='secnot',
	
	license='LPGL, see LICENSE file.',
	
        keywords=['django', 'dsl', 'model', 'field', 'pyparsing'],

	classifiers=[
        'Topic :: Utilities',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Intended Audience :: Developers',
        'Environment :: Web Environment',
        'Framework :: Django',
	'License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)',
        'Development Status :: 3 - Alpha',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3'], 

	# Package
        packages = ['param_field', 'param_field/tests/'],
        install_requires = ['Django', 'pyparsing', 'unittest2'],
	zip_safe = False, 
	include_package_data=True,
)
